/*
 * Proyecto: tregia
 * Fichero: QWriter.prg
 * Descripcin: Generar reportes e informes partiendo de
 * Autores: Juan y Claudio Carramal
 * Fecha: 1/03/2020
 */

#include "Xailer.ch"

CLASS QWriter  //Qtexto

DATA cError      INIT ''           // Msg error (Funciona como Flag)
DATA lValido     INIT .f.          // Por defecto asume texto invlido
DATA nHojaActual INIT -1           // Hoja activa de la Planilla ( -1 ninguna )
DATA lVisible    INIT .f.          // Ver el texto
DATA cTipo       INIT ''           // WRITER / WORD
DATA cTemporal                     // Nombre del archivo temporal
DATA cTempPDF
// Generales: para usar
DATA lPDFgenerado INIT .f. HIDDEN

// Soporte WRITER
DATA oServiceManager                                         HIDDEN
DATA oDesktop                                                HIDDEN
DATA oDocument                                               HIDDEN
method ini_WRITER( cArchivo , lVisible )                     HIDDEN
method reemplazar_WRITER( cBuscado, cReemplazo )             HIDDEN
method tabla_WRITER( cBOOKMARK ,aMatriz , aAnchos, cEstilo ) HIDDEN
method setCelda_WRITER( oCell , uValor, hAlign, vAlign )     HIDDEN
// Soporte WORD
method ini_WORD( cArchivo )                                  HIDDEN
method reemplazar_WORD( cBuscado, cReemplazo )               HIDDEN
method crearGrafico()                                        HIDDEN

access PDFgenerado INLINE ::lPDFgenerado

method New( cTipo , cArchivo , lVisible )
method reemplazar( cBuscado, cReemplazo )
method tabla( ... )  INLINE ::tabla_WRITER( ... )
//method obtenerTablas()
method generarPDF()
method abrirPDF()
method mostrarPDF()
method nombrePDF()


// Graficos Incrustados
method graf_Torta(   cAncla , aDatos , aTitu ) inline ::crearGrafico( cAncla , aDatos , aTitu , 'Torta')
method graf_Torta3D( cAncla , aDatos , aTitu ) inline ::crearGrafico( cAncla , aDatos , aTitu , 'Torta3d')
method graf_XY(      cAncla , aDatos , aTitu ) inline ::crearGrafico( cAncla , aDatos , aTitu , 'DiagramaXY')


ENDCLASS

//------------------------------------------------------------------------------
method New( cTipo , cArchivo , lVisible ) CLASS QWriter
/* Tarea_: Inicializar procesador de texto ( Constructor )
   Recibe: cTipo= 'WORD'   utilizar word de M$ (No implementado)
                  'WRITER' utilizar open document / star office
                   Nul  probar cual est disponible (prioridad WRITER)
*/

local cNomArch, cPATH
::cError:=''                                              // Blanquea Msg error (Funciona como Flag)
::lValido:=.f.                                            // Por defecto asume planilla invlida
::nHojaActual:= -1                                        // Hoja activa de la Planilla ( -1 ninguna )
if valtype(lVisible)='U' ; lVisible:= .t. ;  endif        // Valor por defecto del parametro
::lVisible:=lVisible
if valtype(cTipo)#'C' ; cTipo:='WRITER' ;  endif          // Valor por defecto del parametro
if empty(cTipo)       ; cTipo:='WRITER' ;  endif          //   "    "      "    "      "
cTipo:=alltrim(Upper(cTipo))
// Si se recibe archivo se valida la existencia en disco
if ValType(cArchivo)='U'
  cArchivo:=''
endif
if !Empty(cArchivo)
  // Si el archivo NO tiene path, le agrega el path de DOC ( cambia dbf por doc )
  if At( '\',cArchivo) = 0
    cArchivo:= StrTran( Upper(AppData:sy_dbl_path) , 'DBF' , 'DOC' )+'\'+cArchivo
  endif
  cArchivo:= StrTran( cArchivo , '\\' , '\' )
  if !File( cArchivo )
    ::cError:="No existe el documento de texto " + cArchivo
    ::lValido:=.f.
    return self
  endif
endif
//---- Crea nombre NUEVO del fuente a editar
cPATH  := AppData:sy_ram_path
while .t.
  cNomArch:=cPATH+'_'+str(HB_RandomInt(9999999),7,0)+'.odt'
  cNomArch:=strtran(cNomArch,' ','0')
  if !file(cNomArch); exit; endif
enddo
//
copy file &cArchivo to &cNomArch
cArchivo:=cNomArch
::cTemporal:=cArchivo
// Abre Texto
if cTipo=='WRITER'
  ::ini_WRITER( cArchivo , lVisible )
else
  ::ini_WORD(cArchivo)
endif    // Crea Planilla ODS / XLS
if !::lValido                                             // No pudo Crear: intenta el otro tipo
   if cTipo=='WRITER'; ::ini_WORD(cArchivo); else; ::ini_WRITER(cArchivo); endif // Crea Planilla Excel
endif
if !::lValido ; return nil; endif
return Self

//------------------------------------------------------------------------------
method ini_WRITER( cArchivo , lVisible ) CLASS QWriter
local aProps
::lValido:=.f.
::cTipo:=''
::oServiceManager := TOleAuto():New("com.sun.star.ServiceManager")
::oDesktop := ::oServiceManager:createInstance("com.sun.star.frame.Desktop")
IF ::oDesktop = NIL
  ::cError+='OpenOffice Writter no esta disponible.'
ELSE
  aProps:={}
  if !lVisible
    AAdd( aProps ,  OOProperty( ::oServiceManager , "Hidden", .t. ) )
  endif
  if Empty(cArchivo)
//    ::oDocument := ::oDesktop:loadComponentFromURL("private:factory/swriter","_blank", 0, {})
    ::oDocument := ::oDesktop:loadComponentFromURL("private:factory/swriter","_blank", 0, aProps)
    ::lValido:=.t.
  else
    cArchivo = ConvertToUrl( cArchivo )
//  ::oDocument := ::oDesktop:loadComponentFromURL( cArchivo ,"_blank", 0, {} )
    ::oDocument := ::oDesktop:loadComponentFromURL( cArchivo ,"_blank", 0, aProps )
    ::lValido:=.t.
  endif
ENDIF
if ::lValido
  ::cTipo:='WRITER'
endif
return nil
//------------------------------------------------------------------------------
method ini_WORD( cArchivo ) CLASS QWriter
::lValido:=.f.
::cTipo:=''
::cError:="WORD NO SOPORTADO"
return .f.
//------------------------------------------------------------------------------
method reemplazar_WORD( cBuscado, cReemplazo ) CLASS QWriter
::lValido:=.f.
::cTipo:=''
::cError:="WORD NO SOPORTADO"
return .f.
//------------------------------------------------------------------------------
method reemplazar( cBuscado, cReemplazo ) CLASS QWriter
if ::cTipo=='WRITER'
  return ::reemplazar_WRITER( cBuscado, cReemplazo )
endif
return ::reemplazar_WORD( cBuscado, cReemplazo )
//------------------------------------------------------------------------------
method reemplazar_WRITER( cBuscado, cReemplazo ) CLASS QWriter
local oReplace

oReplace := ::oDocument:createReplaceDescriptor()
oReplace:SearchString  := cBuscado
oReplace:ReplaceString := cReemplazo
::oDocument:replaceAll(oReplace)
return nil
//------------------------------------------------------------------------------
method setCelda_WRITER( oCell , uValor, hAlign, vAlign ) CLASS QWriter
// Metodo para setear el valor en una celda.
// LEFT = 0 , RIGHT = 1 , BLOCK = 2 , CENTER = 3 , STRETCH = 0
local cTipo  := ValType(uValor)
local oTextCell,oCursorCell
local nAjuste

do case
case cTipo='N'
  oCell:setValue( uValor )
  nAjuste:=1 // A la derecha
case cTipo='C'
  oCell:String:= uValor
  nAjuste:=0 // A la izquierda
endcase
oTextCell   := oCell:Text                                       // Texto en el interior de la celda
oCursorCell := oTextCell:createTextCursor()                     // Generamos un cursor de texto
oCursorCell:gotoEND(.t.)                                        // Vamos hasta el final del texto de la celda, indicando que las acciones se aplican a todo el texto de la celda
oCursorCell:ParaAdjust = nAjuste
return nil

//------------------------------------------------------------------------------
method tabla_WRITER( cBOOKMARK ,aMatriz , aSeparadores, aAlineacion, cEstilo ) CLASS QWriter
// Metodo para insertar una planilla en un texto.
// El "lugar" donde se inserta la planilla en el texto lo da cBOOKMARK
// aMatriz contiene los valores a insertar (la matriz). Primer Fila es Titulo
// cEstilo es un tipo de 'decoracion' de la tabla

local oTable,oText,oCursor,oRows,oRow,oCell
local nFilas,nColum,i,j, uValor, nCol, cTipo,nFila
local oMarcas,oAnchor,oBookMark
local aTblColSeps, nAnchoRel, oTextCell, oCursorCell

//----------------------------------------------------- Generacion de Tabla ====
nFilas  := Len(aMatriz)
nColum  := Len(aMatriz[1])
oText   := ::oDocument:getText
oMarcas := ::oDocument:getBookmarks()                // Lista de sealadores (bookmarks)
oAnchor := oMarcas:getByName( cBOOKMARK ):getAnchor()// Obtiene Ancla del Sealador
oCursor := oText:createTextCursor                    // Crea un Cursor
oCursor:gotoRange(oAnchor, .f. )                     // Ubica el cursor en el ancla
oTable  := ::oDocument:createInstance( "com.sun.star.text.TextTable" )  // Crea Tabla
oTable:initialize( nFilas, nColum )                  // Inicializa Tabla
oText:insertTextContent( oCursor, oTable, .F. )      // Inserta la tabla en el cursor actual
oTable:setPropertyValue( "BackTransparent", .F. )                       // Establece Fondo de la Tabla
oTable:setPropertyValue( "BackColor", ( 255 * 256 + 255 ) * 256 + 192 ) // Establece Color Fondo de la Tabla
//---------------------------------------------------- Tratamiento de Filas ====
oRows := oTable:getRows                              // Obtiene Filas
//--------------------------------------------- Primer Fila ( Titulo ) ----
oRow  := oRows:getByIndex( 0 )
oRow:setPropertyValue( "BackTransparent", .F. )                         // Fondo de la Fila
oRow:setPropertyValue( "BackColor", ( 192 * 256 + 192 ) * 256 + 128 )   // Color de fondo de la fila
//----------------------------------------------------- Rellena Matriz ----
for nFila=1 to nFilas
  for nCol=1 to nColum
    uValor := aMatriz[nFila][nCol]
    cTipo  := ValType(uValor)
    oCell  := oTable:getCellByPosition(nCol-1,nFila-1)
    ::setCelda_WRITER( oCell , uValor )
  next
next

//----------------------------------- Ajusta el anchos de las columnas ----
// aSeparadores recibe los % de ancho respecto al ancho de la tabla, donde se
// ubican los separadores de columnas
nAnchoRel  := oTable:TableColumnRelativeSum  // Obtiene ancho relativo de la tabla
aTblColSeps:= oTable:TableColumnSeparators   // Obtiene lista de separadores
j:= Min( Len(aTblColSeps) , Len(aSeparadores) )
for i=1 to j
  aTblColSeps[i]:Position:= nAnchoRel * aSeparadores[i]/100
next
oTable:TableColumnSeparators:=aTblColSeps
return nil

//------------------------------------------------------------------------------
method nombrePDF() CLASS QWriter
return StrTran( ::cTemporal , '.odt','.pdf')

//------------------------------------------------------------------------------
method abrirPDF() CLASS QWriter
// Muestra el PDF solo si fue generado
if ::lPDFgenerado
  ShellExecute( 0, "open", ::nombrePDF() )
endif
return nil

//------------------------------------------------------------------------------
method generarPDF() CLASS QWriter
// Si no se genero el PDF lo genera
local cArchivo
local aProps:={}

if !::lPDFgenerado  // Solo si NO lo genero
  if self:cTipo=='WRITER'
    ::cTempPDF:= convertToURL( alltrim( ::nombrePDF() ) )
    // Prepara parmetros
    aProps:={}
    AAdd( aProps ,  OOProperty( ::oServiceManager , "Overwrite", .t. ) )
    AAdd( aProps ,  OOProperty( ::oServiceManager , "FilterName", "writer_pdf_Export" ) )
    ::oDocument:storeToURL( ::cTempPDF, aProps ) // Salva Como Pdf
    ::oDocument:close(.t.)
    ::lPDFgenerado:=.t.
    //------------------------------------------------------- Guardar WORD ----
  else
  endif
endif
return nil

//------------------------------------------------------------------------------
FUNCTION OOProperty( oServiceManager, cName, uValue )
// Funcion que devuelve un parametro para OpenOffice
LOCAL cStructure
LOCAL oProperty
cStructure := "com.sun.star.beans.PropertyValue"
oProperty := oServiceManager:bridge_GetStruct( cStructure )
oProperty:Name := cName
oProperty:Value := uValue
RETURN oProperty

//------------------------------------------------------------------------------
method mostrarPDF(oCef) CLASS QWriter
// Muestra el PDF solo si fue generado
if ::lPDFgenerado
  if ValType(oCef)='O'
    if oCef:isKindOf('TCefBrowser')
      oCef:Navigate( ::cTempPDF )
    endif
  endif
endif
return nil

//------------------------------------------------------------------------------
function testWriter()
// Esta es una funcion de 'Demo'

local aData, aTitu, aMatriz, aAnchos
local oTX:= QWriter():New(,'QwriterTest.odt',.f.)

if oTX:lValido
  // Reemplazo de texto
  oTX:reemplazar( '[SU NOMBRE]'     , 'estimado seor,' )
  oTX:reemplazar( '[DESDE]'     , DToC( Date()-30 ) )
  oTX:reemplazar( '[HASTA]'     , DToC( Date()    ) )
  // Insercion Tabla
  aMatriz:= { {'--','Enero 20','Febrero 20'} ,;
              {'Prod 1','5','10'}            ,;
              {'Prod 2','12','8'}            ,;
              {'Prod 3','30','15'}    }
  aAnchos:={ 20 ,40 }
  oTX:tabla( 'TABLA' ,aMatriz , aAnchos )

  aData:={ {15} , {20} , {45} } ; aTitu:= { 'Prod 1' , 'Prod 2' , 'Prod 3' }
  oTX:graf_Torta3D( 'TORTA3D',aData, aTitu  )

  // Diagrama XY
  aData:= { {20,30},{01,40},{20,03},{40,03},{03,18},{02,02} }
  aTitu:= {'Costo','Venta'}
  oTX:graf_XY('GRAF_XY',aData, aTitu )

  aData:={ {15} , {20} , {45} } ; aTitu:= { 'Prod 1' , 'Prod 2' , 'Prod 3' }
  oTX:graf_Torta3D( 'TORTA3D',aData, aTitu  )





  // Diagrama Torta
  //aData:= { {60},{40} }
  //oTX:crearGrafico('GRAFICO01',aData, {'Coca','Pepsi'} , 'Torta3d')


  oTX:reemplazar( '[DESDE]'     , DToC( Date()-30 ) )
  oTX:reemplazar( '[HASTA]'     , DToC( Date() ) )

//  oTX:generarPDF()
//  oTX:abrirPDF()


else
  MsgStop(oTX:cError)
endif
return oTX

//------------------------------------------------------------------------------
method crearGrafico( cBOOKMARK , aDatos, aEtiquetas , cTipoGrafico )  CLASS QWriter

local msChartClassID := "12dcae26-281f-416f-a234-c3086127382e"
local oEmbebido, oTexto, diagram, oDATOS, oCursor, oChart ,a
local oAnclas, oANCLA
// Prepara el cursor segun el Ancha / Bookmark
oTexto    := ::oDocument:getText
oAnclas   := ::oDocument:getBookmarks()                            // Lista de sealadores (bookmarks)
oANCLA    := oAnclas:getByName( cBOOKMARK ):getAnchor()            // Obtiene Ancla del Sealador
oCursor   := oTexto:createTextCursor                               // Crea un Cursor
oCursor:gotoRange(oANCLA, .f. )                                    // Ubica el cursor en el ancla
oEmbebido := ::oDocument:createInstance("com.sun.star.text.TextEmbeddedObject")  // Crea Objeto a Incrustar
oEmbebido:setPropertyValue("CLSID", msChartClassID)                // Setea el Tipo (chart)
oTexto:insertTextContent( oCursor, oEmbebido, 0)                   // Inserta el objeto en el texto
oChart    := oEmbebido:getEmbeddedObject()                         // Recupera chart insertado

cTipoGrafico:=AllTrim(Upper(cTipoGrafico))
do case
//------------------------------------------------------------------------------
//-----                         DIAGRAMA X / Y                              ----
//------------------------------------------------------------------------------
case cTipoGrafico= 'DIAGRAMAXY'
  // la cantidad de elementos en la fila aDatos determina la cantidad de variables
  // en aetiquetas van los nombres de las variables.
  diagram := oChart:createInstance("com.sun.star.chart.XYDiagram")   // Crea tipo de diagrama  x/y
  oChart:setDiagram(diagram)
  oDATOS := oChart:getData()
  oDATOS:setColumnDescriptions(aEtiquetas)
  oDATOS:setData(aDatos)

//------------------------------------------------------------------------------
//-----                         TORTA  y  TORTA3D                           ----
//------------------------------------------------------------------------------
case cTipoGrafico$'TORTA-TORTA3D'
  // la cantidad de elementos en la fila aDatos determina la cantidad de variables
  // en aetiquetas van los nombres de las variables.
  diagram := oChart:createInstance("com.sun.star.chart.PieDiagram")   // Crea tipo de diagrama  x/y
  if cTipoGrafico= 'TORTA3D'
    diagram:Dim3D :=   .t.
  endif
  oChart:setDiagram(diagram)
  oDATOS := oChart:getData()
  oDATOS:setRowDescriptions(aEtiquetas)
  oDATOS:setData(aDatos)

endcase
return nil







/*
//------------------------------------------------------------------------------
method verParrafos()  CLASS QWriter

link: https://wiki.openoffice.org/wiki/Documentation/BASIC_Guide/Structure_of_Text_Documents

El ncleo de un documento de texto consiste en una secuencia de prrafos.
Estos no estn nombrados ni indexados y, por lo tanto, no hay forma posible de
acceder directamente a prrafos individuales. Sin embargo, los prrafos se
pueden recorrer secuencialmente con la ayuda del objeto Enumeration descrito en
Introduccin a la API. Esto permite editar los prrafos.
Sin embargo, cuando se trabaja con el objeto Enumeration, se debe tener en
cuenta un escenario especial: no solo devuelve prrafos, sino tambin tablas
(estrictamente hablando, en Apache OpenOffice Writer, una tabla es un tipo
especial de prrafo). Antes de acceder a un objeto devuelto, debe verificar
si el objeto devuelto admite el servicio com.sun.star.text.Paragraph para
prrafos o el servicio com.sun.star.text.TextTable para tablas.
El siguiente ejemplo atraviesa el contenido de un documento de texto en un bucle
y utiliza un mensaje en cada instancia para informar al usuario si el objeto en
cuestin es un prrafo o una tabla.

 Un prrafo a su vez consta de subobjetos individuales. Cada uno de estos
 subobjetos contiene su propia informacin de formato. Si el centro de un
 prrafo, por ejemplo, contiene una palabra impresa en negrita, se representar
 en Apache OpenOffice por tres porciones de prrafo: la porcin antes del tipo
 de negrita, luego la palabra en negrita y finalmente la porcin despus de la
 negrita tipo, que nuevamente se representa como normal.



local Doc          // As Object
local Enum         // As Object
local Enum2        // As Object Para barrer textos dentro del prrafo
local TextElement  // As Object
local TextPortion
local oTextTable
local Rect,RangeAddress,cStructure ,UnRango,oCursor



// Create document object
Doc := ::oDocument
// Create enumeration object

Enum := Doc:Text:createEnumeration()

// loop over all text elements

While Enum:hasMoreElements
  TextElement := Enum:nextElement               // Siguiente elemento
  // Es una tabla
  If TextElement:supportsService("com.sun.star.text.TextTable")
    MsgStop("Tabla !!!.")
  Endif

  // Es un parrafo
  If TextElement:supportsService("com.sun.star.text.Paragraph")

    oCursor := TextElement:createTextCursor                                 // Crea un Cursor que se usar para insertar el objeto


    MsgStop("Parrafo...")
    Enum2 := TextElement:createEnumeration()
    // Barre all sub-paragraphs
    While Enum2:hasMoreElements
      TextPortion := Enum2:nextElement
      MsgStop(  "->" + TextPortion:String  )
    enddo
  EndIf
enddo
return nil

//------------------------------------------------------------------------------
method obtenerTablas() CLASS QWriter
//local aCOMP:={}
local oGraphic, nGraphic, cName,oCompo,oDoc
local oProveedorTablas,oCharts

 com.sun.star.chart

//    'iterate over all charts in a calc document
local oSheet, oDoc, oGrupoSheets
local oCharts
local SheetCount,SheetNo
local ChartCount,ChartNo,oChartModel

oDoc:=::oDocument


oGrupoSheets = oDoc:Sheets:getByIndex(0)


oGrupoSheets := oDoc:getSheets()

SheetCount := oGrupoSheets:Sheets:getCount()
SheetNo    := 0
while ( SheetNo < SheetCount )
  oSheet    := oDoc:Sheets(SheetNo)
  oCharts   := oSheet:Charts
  ChartCount:= oCharts:getCount()
  ChartNo   := 0
  while ( ChartNo<ChartCount )
    oChartModel = oCharts[ChartNo]:EmbeddedObject
//    changeOneChart( oChartModel )
    ChartNo=ChartNo+1
  enddo
  SheetNo=SheetNo+1
enddo
return nil
oProveedorTablas := ::oServiceManager:createInstance("com.sun.star.table.XTableChartsSupplier")
                               com::sun::star::table::XTableCharts getCharts();
oProveedorTablas := ::oServiceManager:createInstance("com.sun.star.table.XTableCharts")
oCharts:= oProveedorTablas:getCharts()  NO
oProveedorTablas := ::oServiceManager:createInstance("com.sun.star.table.XTableChartsSupplier")
oCharts:= oProveedorTablas:getCharts()   NO
oProveedorTablas := ::oServiceManager:createInstance("com::sun::star::table::XTableCharts")
oCharts:= oProveedorTablas:getCharts()  NO
oGraphic := ::oDocument:getGraphicObjects()
// oGraphic := ::oDocument:getTableChart() // NO
FOR nGraphic := 1 TO oGraphic:getCount()
  oGraphic := oGraphic:getByIndex( nGraphic - 1 )
  cName := Lower( oGraphic:getName() )
//nMerge := AScan( aMergue, { | e | e[ 1 ] == cName })
//IF nMerge > 0
//   oGraphic:graphicURL := OOUrlFile( aMerge[ nMerge, 2 ] )
//ENDIF
NEXT
aCOMP:= ::oDesktop:getComponents()
if aCOMP:hasElements()
  MsgStop('SI, hay componentes')
else
  MsgStop('No hay componentes')
endif
return nil
*/